// =====================================================================================================================
//  fennec, a free and open source game engine
//  Copyright © 2025  Medusa Slockbower
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <https://www.gnu.org/licenses/>.
// =====================================================================================================================

/// \file __type_traits.h
/// \internal

#ifndef FENNEC_LANG_DETAIL_TYPE_TRAITS_H
#define FENNEC_LANG_DETAIL_TYPE_TRAITS_H

#include <fennec/lang/constants.h>
#include <fennec/lang/type_traits.h>
#include <fennec/lang/float.h>

namespace fennec
{

namespace detail
{

template<typename> struct __is_void_helper       : false_type {};
template<>         struct __is_void_helper<void> : true_type {};

template<typename> struct __is_bool_helper         : false_type {};
template<>         struct __is_bool_helper<bool_t> : true_type {};

template<typename> struct __is_integral_helper           : false_type {};
template<>         struct __is_integral_helper<bool_t>   : true_type {};
template<>         struct __is_integral_helper<char_t>   : true_type {};
template<>         struct __is_integral_helper<char8_t>  : true_type {};
template<>         struct __is_integral_helper<char16_t> : true_type {};
template<>         struct __is_integral_helper<char32_t> : true_type {};
template<>         struct __is_integral_helper<schar_t>  : true_type {};
template<>         struct __is_integral_helper<uchar_t>  : true_type {};
template<>         struct __is_integral_helper<wchar_t>  : true_type {};
template<>         struct __is_integral_helper<short_t>  : true_type {};
template<>         struct __is_integral_helper<ushort_t> : true_type {};
template<>         struct __is_integral_helper<int_t>    : true_type {};
template<>         struct __is_integral_helper<uint_t>   : true_type {};
template<>         struct __is_integral_helper<long_t>   : true_type {};
template<>         struct __is_integral_helper<ulong_t>  : true_type {};
template<>         struct __is_integral_helper<llong_t>  : true_type {};
template<>         struct __is_integral_helper<ullong_t> : true_type {};

template<typename TypeT> struct __is_signed_helper   : bool_constant<TypeT(-1) < TypeT(0)> {};
template<typename TypeT> struct __is_unsigned_helper : bool_constant<TypeT(-1) >= TypeT(0)> {};

template<typename> struct __is_floating_point_helper           : false_type {};
template<>         struct __is_floating_point_helper<float_t>  : true_type {};
template<>         struct __is_floating_point_helper<double_t> : true_type {};

}

}

#endif // FENNEC_LANG_DETAIL_TYPE_TRAITS_H
